import type { PluginInterface } from '@ckeditor/ckeditor5-core/src/plugin';
import type { Command } from 'ckeditor5/src/core';
import type { ButtonExecuteEvent } from 'ckeditor5/src/ui';
import type { Locale } from 'ckeditor5/src/utils';
import type { AccordionItemInsertPosition } from './bootstrapaccordiontypes';
import { Plugin } from 'ckeditor5/src/core';
import { ButtonView } from 'ckeditor5/src/ui';
import { WidgetToolbarRepository } from 'ckeditor5/src/widget';
import icon from '../../../icons/bootstrapaccordion.svg';

/**
 * Defines the user interface for editing Bootstrap Accordion widgets.
 */
export default class BootstrapAccordionUI extends Plugin implements PluginInterface {

  /**
   * The plugin's name in the PluginCollection.
   */
  static get pluginName(): 'BootstrapAccordionUI' {
    return 'BootstrapAccordionUI' as const;
  }

  /**
   * The plugin's dependencies.
   */
  static get requires() {
    return [WidgetToolbarRepository] as const;
  }

  /**
   * @inheritdoc
   */
  init() {
    const { editor } = this;
    const { commands, ui } = editor;
    const { componentFactory } = ui;

    componentFactory.add('bootstrapAccordion', locale =>
      createToolbarButton(locale, locale.t('Accordion'), icon, commands.get('insertBootstrapAccordion')!));
    componentFactory.add('bootstrapAccordionInsertItemAbove', locale =>
      createToolbarButton<AccordionItemInsertPosition>(locale, locale.t('Insert item above'), null, commands.get('insertBootstrapAccordionItem')!, 'before'));
    componentFactory.add('bootstrapAccordionInsertItemBelow', locale =>
      createToolbarButton<AccordionItemInsertPosition>(locale, locale.t('Insert item below'), null, commands.get('insertBootstrapAccordionItem')!, 'after'));
    componentFactory.add('bootstrapAccordionRemoveItem', locale =>
      createToolbarButton(locale, locale.t('Delete item'), null, commands.get('removeBootstrapAccordionItem')!));  
  }

  /**
   * @inheritdoc
   */
  afterInit() {
    const editor = this.editor;
    const widgetToolbarRepository = editor.plugins.get(WidgetToolbarRepository);
    widgetToolbarRepository.register('bootstrapAccordion', {
      items: ['bootstrapAccordionInsertItemAbove', 'bootstrapAccordionInsertItemBelow', 'bootstrapAccordionRemoveItem'],
      getRelatedElement: selection => {
        const accordion = selection.focus?.getAncestors()
          .find(node => node.is('element') && node.hasClass('ckeditor5-bootstrap-accordion__widget'));
        return accordion?.is('element') ? accordion : null;
      }
    });
  }

}

/**
 * Creates a button.
 *
 * @param locale
 *   The locale.
 * @param label
 *   The button's label.
 * @param icon
 *   The button's icon (optional).
 * @param className
 *   The button's class (optional).
 * @param withText
 *   Set to force text display even if the button has an icon.
 * @returns
 *   A button with the specified parameters.
 */
function createButton(locale: Locale, label: string, icon?: string | null, className?: string | null, withText?: boolean | string | null): ButtonView {
  const button = new ButtonView(locale);
  button.set({
    label: typeof withText === 'string' ? withText : label,
    icon,
    tooltip: icon ? label : false,
    withText: withText || !icon,
    class: className
  });
  return button;
}

/**
 * Creates a toolbar button to execute a command.
 *
 * @param locale
 *   The locale.
 * @param label
 *   The button's label.
 * @param icon
 *   The button's icon (can be `null` for none).
 * @param command
 *   The command to execute when pressing the button.
 * @param value
 *   The value to pass to the command (optional).
 * @returns
 *   A button with the specified parameters.
 */
function createToolbarButton<T>(locale: Locale, label: string, icon: string | null, command: Command, value?: T): ButtonView {
  const editor = command.editor, buttonView = createButton(locale, label, icon);
  // Displays the tooltip on hover if there is no icon.
  buttonView.tooltip = !!icon;
  // Disables the button if the command is disabled.
  buttonView.bind('isEnabled').to(command);
  // Executes the command with the button's value on click.
  buttonView.on<ButtonExecuteEvent>('execute', () => {
    command.execute({ value });
    editor.editing.view.focus();
  });
  return buttonView;
}
